// ChkFileVer.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

int main(int argc, char* argv[])
{
	LPCTSTR lpszFile1 = (LPTSTR)NULL;
	LPCTSTR lpszFile2 = (LPTSTR)NULL;
	LPCTSTR lpszCopy = (LPTSTR)NULL;
	
	//////////////////////////////////////////////
	// Get args
	//
	if (argc >= 3)
	{
		lpszFile1 = argv[1];
		lpszFile2 = argv[2];
		if (argc == 4)
		{
			lpszCopy = argv[3];
		}
	}
	else
	{
		ShowMsg(_T("Usage: chkfilever file1 file2 [/copy]\n\rReturn values:\n\r  - 1 if file1 is newer\n\r  - 2 if fname2 is newer\n\r  - 3 if they are the same version\n\r  - 0 if there is an error\n\r"),
			MB_OK|MB_ICONASTERISK);
		return 0;
	}
	
	//////////////////////////////////////////////
	// Check result
	//
	if (lpszFile1 == (LPCTSTR)NULL || lpszFile2 == (LPCTSTR)NULL)
	{
		ShowErrorMsg(ERROR_BAD_ARGUMENTS);
		return 0;
	}

	//////////////////////////////////////////////
	// Check File1
	//
	if (GetFileAttributes(lpszFile1) == 0xFFFFFFFF)
	{
		ShowErrorMsg(ERROR_FILE_NOT_FOUND);
		return 0;
	}
	else
	{
		//////////////////////////////////////////////
		// Check File2
		//
		if (GetFileAttributes(lpszFile2) == 0xFFFFFFFF)
		{
			ShowErrorMsg(ERROR_FILE_NOT_FOUND);
			return 0;
		}
	}
	
	//////////////////////////////////////////////
	// Check 
	//   verson(File1) <?> verson(File2) or
	//   LastWriteTime(File1) <?> LastWriteTime(File2)
	//
	int nRetVal = WhichIsNewer(lpszFile1, lpszFile2);

	//////////////////////////////////////////////
	// Need copy File1 -> File2 ?
	if (nRetVal == 1 && lpszCopy != (LPTSTR)NULL)
	{
		if (!CopyFile(lpszFile1, lpszFile2))
		{
			// Copy error...
			ShowErrorMsg(GetLastError());
			nRetVal = 0;
		}
	}
	return nRetVal;
}

/////////////////////////////////////////////////
// WhichIsNewer
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// return values:
//	1 if fname1 is newer
//	2 if fname2 is newer
//	3 if they are the same version
//	0 if there is an error
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int WhichIsNewer(LPCTSTR lpszFile1, LPCTSTR lpszFile2)
{
	int ndxNewerFile = 0;
	VS_FIXEDFILEINFO vsf1, vsf2;
	memset(&vsf1, 0, sizeof(VS_FIXEDFILEINFO));
	memset(&vsf2, 0, sizeof(VS_FIXEDFILEINFO));
	if (GetFileVersion(lpszFile1, &vsf1) == NO_ERROR 
			&& GetFileVersion(lpszFile2, &vsf2) == NO_ERROR)
	{
		// both files have a file version resource
		// compare by file version resource
		if (vsf1.dwFileVersionMS > vsf2.dwFileVersionMS)
		{
			ndxNewerFile = 1;
		}
		else
		{
			if (vsf1.dwFileVersionMS < vsf2.dwFileVersionMS) 
			{
				ndxNewerFile = 2;
			}
			else 
			{   // if (vsf1.dwFileVersionMS == vsf2.dwFileVersionMS)
				if (vsf1.dwFileVersionLS > vsf2.dwFileVersionLS)
				{
					ndxNewerFile = 1;
				}
				else if (vsf1.dwFileVersionLS < vsf2.dwFileVersionLS)
				{
					ndxNewerFile = 2;
				}
				else 
				{   // if (vsf1.dwFileVersionLS == vsf2.dwFileVersionLS)
					ndxNewerFile = 3;
				}
			}
		}
	}
	else
	{
		// compare by date
		DWORD dwRetCode = NO_ERROR;
		FILETIME ft1, ft2;
		memset(&ft1, 0, sizeof(FILETIME));
		memset(&ft2, 0, sizeof(FILETIME));
		if ((dwRetCode = GetFileDate(lpszFile1, &ft1)) == NO_ERROR
				&& (dwRetCode = GetFileDate(lpszFile2, &ft2)) == NO_ERROR)
		{
			LONG x = CompareFileTime(&ft1,&ft2);
			if (x == -1) 
				ndxNewerFile = 2;
			else if (x == 0) 
				ndxNewerFile = 3;
            else if (x == 1)
				ndxNewerFile = 1;
			else
			{
				ShowErrorMsg(ERROR_INVALID_FUNCTION);
				return 0;
			}
		}
		else
		{
			ShowErrorMsg(dwRetCode);
			return 0;
		}
	}
	return ndxNewerFile;
}

// This function gets the file version info structure
DWORD GetFileVersion(LPCTSTR lpszFileName, VS_FIXEDFILEINFO *pvsf)
{
	DWORD dwErr = NO_ERROR;
	DWORD dwHandle;
	DWORD cchver = GetFileVersionInfoSize((LPTSTR)lpszFileName, &dwHandle);
	if (cchver == 0)
	{
		dwErr = GetLastError();
		if (dwErr == NO_ERROR)
			return ERROR_CALL_NOT_IMPLEMENTED;
		else
			return dwErr;
	}
	_TCHAR* pver = new _TCHAR[cchver];
	BOOL bret = GetFileVersionInfo((LPTSTR)lpszFileName, dwHandle, cchver, pver);
	if (!bret) 
	{
		dwErr = GetLastError();
		if (dwErr == NO_ERROR)
			return ERROR_CALL_NOT_IMPLEMENTED;
		else
			return dwErr;
	}
	UINT uLen;
	void *pbuf;
	bret = VerQueryValue(pver, _T("\\"), &pbuf, &uLen);
	if (!bret) 
	{
		dwErr = GetLastError();
		if (dwErr == NO_ERROR)
			return ERROR_CALL_NOT_IMPLEMENTED;
		else
			return dwErr;
	}
	memcpy(pvsf, pbuf, sizeof(VS_FIXEDFILEINFO));
	delete[] pver;
	return NO_ERROR;
}

DWORD GetFileDate(LPCTSTR lpszFileName, LPFILETIME lpFt)
{
	DWORD dwRetCode = NO_ERROR;
	WIN32_FILE_ATTRIBUTE_DATA wfad;
	memset(&wfad, 0, sizeof(WIN32_FILE_ATTRIBUTE_DATA));
	if (GetFileAttributesEx(lpszFileName, GetFileExInfoStandard, &wfad))
	{
		lpFt->dwHighDateTime = wfad.ftLastWriteTime.dwHighDateTime;  
		lpFt->dwLowDateTime = wfad.ftLastWriteTime.dwLowDateTime;  
	}
	else
	{
		dwRetCode = ::GetLastError();
	}
	return dwRetCode;
}

VOID ShowErrorMsg(DWORD dwErrCode)
{
	LPVOID lpMsgBuf = NULL;
	::FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM, 
		NULL,
		dwErrCode,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&lpMsgBuf,
		0,
		NULL);
	if (lpMsgBuf != NULL)
	{
		ShowMsg((LPCTSTR)lpMsgBuf, MB_OK|MB_ICONERROR);
		::LocalFree(lpMsgBuf);
	}
}

VOID ShowMsg(LPCTSTR lpszMsg, int nType)
{
//	::MessageBox(NULL, lpszMsg, _T("ChkFileVer:"), nType);
	_tprintf(lpszMsg);
}

BOOL CopyFile(LPCTSTR lpszFrom, LPCTSTR lpszTo)
{
	return ::CopyFile(lpszFrom, lpszTo, FALSE);
}
